import java.io.IOException;
import java.util.HashMap;
import java.util.ArrayList;
import greenfoot.World;

/**
 * LocalClient
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class LocalClient extends GameClient
{
    private HashMap<Integer, PlayerShip> players = new HashMap<Integer, PlayerShip>();
    private HashMap<Integer, Missile> missiles = new HashMap<Integer, Missile>();
    private ArrayList<Missile> missilesWithoutIdentity = new ArrayList<Missile>();
    
    public LocalClient(String ip, int port) throws IOException
    {
        super(ip, port);
    }
    
    @Override
    public void setLocation(int x, int y)
    {
        super.setLocation(x, y);
        printLine("L" + x + ":" + y + ";");
    }
    
    public void newMissile(Missile missile)
    {
        printLine("O" + missile.getX() + ":" + missile.getY() + ";");
        missilesWithoutIdentity.add(missile);
    }
    
    public void act(World world)
    {
        String line = readLine();
        if (line != null) {
            Integer playerID = null;
            while (!line.isEmpty()) {
                int semicolonIndex = line.indexOf(';');
                int colonIndex, x, y;
                Missile missile;
                switch (line.charAt(0)) {
                    
                    // Obtain my identity
                    case 'I':
                        identity = Integer.parseInt(line.substring(1, semicolonIndex));
                        break;
                    
                    // Indicate Player
                    case 'P':
                        playerID = new Integer(Integer.parseInt(line.substring(1, semicolonIndex)));
                        if (playerID != identity && players.get(playerID) == null)
                            players.put(playerID, new PlayerShip(false));
                        break;
                    
                    // New location for indicated Player
                    case 'L':
                        if (playerID == identity)
                            break;
                        colonIndex = line.indexOf(':');
                        x = Integer.parseInt(line.substring(1, colonIndex));
                        y = Integer.parseInt(line.substring(colonIndex + 1, semicolonIndex));
                        PlayerShip ship = players.get(playerID);
                        if (ship.getWorld() == null)
                            world.addObject(ship, x, y);
                        else
                            ship.setLocation(x, y);
                        break;
                    
                    // Object
                    case 'O':
                        colonIndex = line.indexOf(':');
                        int objectID = Integer.parseInt(line.substring(1, colonIndex));
                        int nextStartIndex = colonIndex + 1;
                        switch (line.charAt(nextStartIndex)) {
                            
                            // Create Object
                            case 'C':
                                // Don't create Object if we are the one that originated it
                                if (playerID == identity)
                                    break;
                                
                                colonIndex = line.indexOf(':', colonIndex + 1);
                                x = Integer.parseInt(line.substring(nextStartIndex + 1, colonIndex));
                                y = Integer.parseInt(line.substring(colonIndex + 1, semicolonIndex));
                                missile = new Missile();
                                missiles.put(new Integer(objectID), missile);
                                world.addObject(missile, x, y);
                                break;
                            
                            // Remove Object
                            case 'R':
                                missile = missiles.remove(new Integer(objectID));
                                if (missile != null)
                                    world.removeObject(missile);
                        }
                    break;
                    
                    // Obtain an identifier for Object we created
                    case 'o':
                        missiles.put(new Integer(Integer.parseInt(line.substring(1, semicolonIndex))), missilesWithoutIdentity.remove(0));
                    break;
                    
                    
                }
                line = line.substring(semicolonIndex + 1);
            }
        }
    }
    
    
    
}