import greenfoot.World;
import greenfoot.GreenfootImage;
import greenfoot.Actor;
import java.io.IOException;
import java.awt.Point;
import java.awt.Color;

/**
 * Screen
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class Screen extends World
{
    private static final int WIDTH  = 400;
    private static final int HEIGHT = 600;
    public static Screen main;
    static
    {
        GUI_Component.setDefaultColors(new Color(0, 0, 0, 180), Color.WHITE, Color.WHITE, new Color(70, 70, 70, 180), Color.GRAY, Color.BLUE);
    }
    private static final int SEND_MY_LOCATION_RATE = 50;

    private PlayerShip myShip = new PlayerShip(true);
    private LocalClient client;
    private GameServer server;
    
    private boolean securityManager;
    
    private Button newGameBtn = new Button("New Game", new Point(150, 23));
    
    private Container setupContainer = new Container(new Point(1, 4));
    private CheckBoxLabel hostChkLbl = new CheckBoxLabel("Host Server?", false);
    private Container ipContainer = new Container(new Point(2, 1));
    private TextBox ipInput = new TextBox(new Point(120, 25), "localhost");
    private TextBox portInput = new TextBox(new Point(120, 25), "7834");
    private Button exitSetupBtn = new Button("Exit", new Point(50, 23));
    private Button connectBtn = new Button("Enter", new Point(50, 23));
    
    public Screen()
    {
        super(WIDTH, HEIGHT, 1, false);
        main = this;
        
        GreenfootImage back = new GreenfootImage(WIDTH, HEIGHT);
        back.fill();
        setBackground(back);
        
        if (System.getSecurityManager() != null) {
            addObject(new Actor(){{ setImage(new GreenfootImage("  Security Manager  \n  Present  ", 20, Color.WHITE, Color.BLACK)); }}, WIDTH / 2, HEIGHT / 2);
            securityManager = true;
            return;
        }
        
        setupContainer.addComponent(hostChkLbl);
        ipContainer.addComponent(new Label("IP Address:"));
        ipContainer.addComponent(ipInput);
        setupContainer.addComponent(ipContainer);
        Container portContainer = new Container(new Point(2, 1));
        portContainer.addComponent(new Label("Port:"));
        portInput.acceptOnly(TextBox.NUMBERS);
        portContainer.addComponent(portInput);
        portContainer.justifyHorizontally(WindowComponent.END);
        setupContainer.addComponent(portContainer);
        Container btnConainer = new Container(new Point(2, 1));
        btnConainer.addComponent(exitSetupBtn);
        btnConainer.addComponent(connectBtn);
        setupContainer.addComponent(btnConainer);
        
        
        addObject(newGameBtn, WIDTH / 2, HEIGHT / 2);
        
//         boolean host = JOptionPane.showConfirmDialog(null, "Host Server?", "", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
//         
//         String ip = null;
//         if (host)
//             ip = "localhost";
//         else
//             while (ip == null)
//                 ip = JOptionPane.showInputDialog("IP address?");
//         
//         int port;
//         while (true)
//             try {
//                 port = Integer.parseInt(JOptionPane.showInputDialog("Port number?"));
//                 break;
//             } catch (Exception e) {}
//         
//         if (host)
//             try {
//                 server = new GameServer(port);
//             } catch (IOException e) {}
//         try {
//             client = new LocalClient(ip, port);
//         } catch (IOException e) {}
//         
//         
//         addObject(myShip, WIDTH / 2, HEIGHT - 50);
    }
    
    private long lastLocationSentAt = System.currentTimeMillis();
    
    @Override
    public void act()
    {
        if (securityManager)
            return;
        
        if (newGameBtn.wasClicked())
        {
            removeObject(newGameBtn);
            addObject(setupContainer, WIDTH / 2, HEIGHT / 2);
        }
        
        if (hostChkLbl.hasChanged())
            ipContainer.hide(hostChkLbl.isChecked());
        
        if (exitSetupBtn.wasClicked())
        {
            setupContainer.removeFromWorld();
            addObject(newGameBtn, WIDTH / 2, HEIGHT / 2);
        }
        
        if (setupContainer.inWorld())
        {
            int port = (portInput.getText().isEmpty() ? -1 : Integer.parseInt(portInput.getText()));
            
            connectBtn.setEnable(port != -1);
            if (connectBtn.wasClicked())
            {
                boolean host = hostChkLbl.isChecked();
                
                String ip = null;
                if (host)
                    ip = "localhost";
                else
                    ip = ipInput.getText();
                
                
                if (host)
                    try {
                        server = new GameServer(port);
                    } catch (IOException e) {
                        System.out.println("Failed");
                        return;
                    }
                try {
                    client = new LocalClient(ip, port);
                } catch (IOException e) {
                    System.out.println("Failed");
                    return;
                }
                
                setupContainer.removeFromWorld();
                addObject(myShip, getPlayerStartX(), getPlayerStartY());
            }
        }
        
        if (client == null)
            return;
        
        long currentTime = System.currentTimeMillis();
        
        if (currentTime - lastLocationSentAt >= SEND_MY_LOCATION_RATE && myShip.hasMoved()) {
            client.setLocation(myShip.getX(), myShip.getY());
            lastLocationSentAt = currentTime;
        }
        client.act(this);
    }
    
    public void shootMissile(Missile missile)
    {
        client.newMissile(missile);
    }
    
    public int getPlayerStartX()
    {
        return WIDTH / 2;
    }
    public int getPlayerStartY()
    {
        return HEIGHT - 50;
    }
}