import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.io.IOException;
import java.util.ArrayList;

/**
 * Server
 * 
 * @author Taylor Born
 * @version January 2014
 */
public abstract class Server implements Runnable
{
    private volatile ServerSocket serverSocket;
    protected ArrayList<ClientInfoCopy> clients = new ArrayList<ClientInfoCopy>();
    private volatile ArrayList<Socket> newSocketBuffer = new ArrayList<Socket>();
    private volatile boolean closing;

    protected Server(int port) throws IOException
    {
        serverSocket = new ServerSocket(port);
        serverSocket.setSoTimeout(1000);

        (new Thread(new AcceptClientsThread())).start();
        
        Thread serverExecutionThread = new Thread(this);
        serverExecutionThread.setPriority(Thread.MAX_PRIORITY);
        serverExecutionThread.start();
    }
    
    public void close()
    {
        closing = true;
    }
    
    @Override
    public final void run()
    {
        while (true) {
            if (closing)
                return;
            synchronized (newSocketBuffer) {
                while (!newSocketBuffer.isEmpty())
                {
                    ClientInfoCopy client = new ClientInfoCopy(newSocketBuffer.remove(0));
                    clients.add(client);
                    newClientAdded(client);
                }
            }
            act();
        }
    }
    
    protected void newClientAdded(ClientInfoCopy client)
    {}
    
    protected abstract void act();
    
    private class AcceptClientsThread implements Runnable
    {
        @Override
        public void run()
        {
            while (true) {
                if (closing)
                    return;
                try {
                    Socket socket;
                    try {
                        socket = serverSocket.accept();
                    }
                    catch (SocketTimeoutException e) {
                        continue;
                    }
                    synchronized (newSocketBuffer) {
                        newSocketBuffer.add(socket);
                    }
                }
                catch (IOException e) {}
            }
        }
    }
    
    @Override
    public void finalize() throws Throwable
    {
        super.finalize();
        serverSocket.close();
    }
}